
import peasy.*;
import traer.physics.*;
import java.util.*;
import controlP5.*;


//Book info, hashed by bib#
Table bibTable;
Table checkoutTable;
Table categoryTable;
Hashtable<Integer, Book> books = new Hashtable<Integer, Book>();
Hashtable<Integer, Book> booksById = new Hashtable<Integer, Book>();
ArrayList<Checkout> allCheckouts = new ArrayList<Checkout>();
int bibRowNums;
int checkoutRowNums;
int categoryRowNums;

//overlay controls
ControlP5 cp5;
int v1;
DropdownList d1;
Toggle camButton;

//Values
float DRAG_COEFF = 0;
float GRAVITY = 0;

int NUM_PARTICLES = 80000;
final float PARTICLE_MASS = 3;

String [] categoryLabels = {"Anti-Family", "Cultural Insensitivty", "Drugs/Alcohol/Smoking", "Gambling", "Langauge", "Sex Education", "Sexually Explicit",
"Unsuited for Age Group", "Violence", "Racism", "Homosexuality", "Political Viewpoint", "Religious Viewpoint", "Nudity", "Occult/Satanism", "Suicide", "Sexism"};

//Camera and Particle System
PeasyCam cam;
ParticleSystem ps;
ArrayList<Node> nodes = new ArrayList();
Boolean camControls = false;

//Presentation vars
Boolean presentationMode = true;
int presentationWait = 40000;
int currTime;

int currBook = 1;
String currTitle = "";
//
final float MIN_RADIUS = 10;
final float MAX_RADIUS = 100;      // 50
final float ROTATION_SPEED = 0.01; // radians

void initializeOverlay(){
   colorMode(RGB);
   cp5 = new ControlP5(this);
   cp5.addSlider("alphaSlider")
     .setPosition(50, 60)
     .setSize(200, 20)
     .setRange(1, 255)
     .setValue(10)
     .setColorCaptionLabel(color(255,255,255))
     .setCaptionLabel("Alpha");
   cp5.setAutoDraw(false);
   d1 = cp5.addDropdownList("bookTitleDropdown").setPosition(1190, 60).setSize(200, 75);
   d1.setBackgroundColor(color(180, 20, 20));
   println("Books size: " + booksById.values().size());
   for(int i=0; i<booksById.values().size(); i++){
     d1.addItem(booksById.get(i+1).title(), i);
   }
   d1.setBarHeight(20);
   d1.setValue(2);
   d1.setCaptionLabel("Book Title");
   d1.setOpen(false);
   
   CColor col = new CColor();
   col.setActive(color(0, 200, 100));
   
   camButton = cp5.addToggle("camController")
      .setPosition(125, 10)
      .setSize(25, 25)
      .setColor(col)
      .setCaptionLabel("Camera Control")
      .setColorBackground(color(200, 0, 0))
      .setColorActive(color(0, 200, 100));
  
   colorMode(HSB);
   
}

void bookTitleDropdown(int dropdownNum){
  //println("Drop:" + dropdownNum);
  currBook = dropdownNum +1;
  initializeParticles();
}



void alphaSlider(int alphaVal){
  ALPHA_VAL = alphaVal;
}

void camController(int val){
 camControls = !camControls;
 println(camControls);
 cam.setMouseControlled(camControls);
}


void setup(){
  println("Setting up...");
  size(1280, 720, P3D);
  pixelDensity(2);
  
  System.setProperty("user.dir",  dataPath(""));
  System.out.println("Working Directory = " +
              System.getProperty("user.dir"));
  
  cam = new PeasyCam(this, 200);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(10000);
  cam.setState(getInitialState(), 4000);
  cam.setMouseControlled(false);
  

  
  
  ps = new ParticleSystem( GRAVITY, DRAG_COEFF );
  ps.setIntegrator(ParticleSystem.MODIFIED_EULER);
  
  currTime = millis();
  
  smooth();
  colorMode( HSB );
  initializeData();
  NUM_PARTICLES = allCheckouts.size();
  initializeParticles();
  initializeOverlay();
}

/**
* Initializa and parse all data 
**/
void initializeData(){
  
  
  bibTable = loadTable("bib_numbers.csv", "header");
  bibRowNums = bibTable.getRowCount();
  println("BibNumber Rows: " + bibRowNums);
  
  
  categoryTable = loadTable("categories.csv", "csv");
  categoryRowNums = categoryTable.getRowCount();
  println("Category Rows:"  + categoryRowNums);

  
  for(int i=0; i<bibRowNums; i++){
    Integer id = bibTable.getInt(i,0); 
    Integer bibNumber = bibTable.getInt(i, 1);
    String title = bibTable.getString(i, 2);
    Book newBook = new Book(id, title);
    String[] cats = categoryTable.getString(id-1, 0).split(" "); 
     for (int j=0; j<cats.length; j++){
      newBook.addCategory(new Integer(cats[j]));
    }
    books.put(bibNumber, newBook); 
    booksById.put(id, newBook);
  }
  
  
  checkoutTable = loadTable("checkouts.csv", "header");
  checkoutRowNums = checkoutTable.getRowCount();
  println("Checkout Rows:"  + checkoutRowNums);
  
  for(int i=0; i<checkoutRowNums; i++){
    Integer bibNumber = checkoutTable.getInt(i, 3); 
    String cout = checkoutTable.getString(i,1);
    String cin = checkoutTable.getString(i, 2);
    Book currBook = books.get(bibNumber);
    Checkout newCheckout = new Checkout(cout, cin, currBook);
    allCheckouts.add(newCheckout);
    currBook.addCheckout(newCheckout);
  } 
  
  
}

/*
* Create, initialize, and position all particles
*/
void initializeParticles(){
  background(0);
  
  ps.clear();
  nodes.clear();
  
  for (int i=0; i < NUM_PARTICLES; i++){
    
    if(allCheckouts.get(i).book().id() == currBook){
    currTitle = allCheckouts.get(i).book().title();
    Particle p = ps.makeParticle( PARTICLE_MASS,
    200,
    allCheckouts.get(i).getCheckoutTimeOfDay(),
    allCheckouts.get(i).getCheckoutDate());
    
    
    
    float dX = 700;
    float dY = allCheckouts.get(i).getCheckinTimeOfDay();
    float dZ = allCheckouts.get(i).getCheckinDate();
    
   // println("checkout date: " + allCheckouts.get(i).getCheckoutDate() + "checkin date:" + allCheckouts.get(i).getCheckinDate()); //<>//

    Node n = new Node(p, allCheckouts.get(i).book().getCategories(), allCheckouts.get(i).duration(), dX + 500, dY, dZ);
    nodes.add(n);
    }
  }
  println("Particles: " + nodes.size());

}

void drawText(){
  textMode(SHAPE);
  smooth();
  textSize(32);
  for(int i=2006; i<= 2015; i ++){
    colorMode(RGB);
    color c = color( 255, 255, 255, 255);
    stroke(c);
    fill(c);
    text(Integer.toString(i),200, 720, (i-2006) * 365); 
    text(Integer.toString(i),1200, 720, (i-2006) * 365); 
    color b = color( 180, 180, 180, 255);
    stroke(b);
    line( 200, 720, (i-2006) * 365, 1200, 720, (i-2006) * 365);  
    line( 200, 1440, (i-2006) * 365, 1200, 1440, (i-2006) * 365);  
  }
  textMode(MODEL);
  colorMode(HSB);
  
}

void drawLegend(){
    for (int i=0; i<17; i++){
        textMode(SHAPE);
      colorMode(HSB);  
      color c = color((int)(255 / 17.0) * i, 255, 255, 255);
      fill(c);
      rect(20, 315+(i*10), 10, 10);
      colorMode(RGB);
      textSize(10);
      textAlign(LEFT, BOTTOM);
      fill(color(255, 255, 255, 255));
      text(categoryLabels[i],35,325+(i*10));
     
    }
    /*textAlign(CENTER, CENTER);
    textSize(32);
    text("Frequejntly Challenged Titles", 720, 50);*/
    colorMode(HSB);
    textMode(MODEL);
    
}

void addSignature(){
  textMode(SHAPE);
  smooth();
  textSize(12);
  colorMode(RGB);
  color c = color(255);
  stroke(c);
  fill(c);
  text("Frequently Challenged Titles",5, height - 36, 0); 
  text(currTitle,5, height - 24, 0); 
  text("Seattle Public Library Data",5, height - 12, 0); 
  text("Jordan Hughes",width - 200, height - 36, 0); 
  text("Computer Science, MS", width - 200, height - 24);
  text("UCSB 2016",width - 200, height - 12, 0); 
  textMode(MODEL);
  colorMode(HSB);
}

/**
* Drawing loop
**/
void draw(){
  
  if(millis() - currTime > 4000 && presentationMode){
     points = false; 
  }
  
  if(millis() - currTime >= presentationWait && presentationMode){
    points = true;
    cam.setState(getRandomState(), 4000);
    bookTitleDropdown((int)random(0, 45));
    currTime = millis();
  }
  
  if(points){
    background(0);
  }
  
  for( int i=0; i<nodes.size(); i++ ){
    Node n = nodes.get(i);
    n.draw();
  }
  ps.tick();
  

  drawText();
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  if(points){
    drawLegend(); 
  }
  if(!presentationMode){
    colorMode(RGB);
    noStroke();
    fill(color(100, 100, 100, 200));
    rect(0, 0, 1440, 100); 
  
    cp5.draw();
  }else{
    addSignature();
  }
    cam.endHUD();
  
  colorMode(HSB);
  hint(ENABLE_DEPTH_TEST);


  
  //noStroke();
  //fill(200, 20);
}